/* File.....: progress.prg
 * Author...: Peter Townsend
 * CIS ID...:
 * e-mail...: cephas@iname.com
 *
 * A set of functions to display a progress bar during an operation.
 *     INIT_PROGRESS()
 *     PROGRESS()
 *     KILL_PROGRESS()
 *
 */
/*
 * This is an original work by Peter Townsend and is
 * hereby placed in the public domain.
 */
#include "common.ch"
#include "box.ch"
#define PRO_ROW 1
#define PRO_TTL 2
#define PRO_DUN 3
#define PRO_SCR 4
#define PRO_C_BOX "w+/bg"
#define PRO_C_LYN "rb/rb"
* ======================================================================== *
/*  $DOC$
 *  $FUNCNAME$
 *      INIT_PROGRESS()
 *
 *  $ONELINER$
 *      Initialise progress display data.
 *
 *  $SYNTAX$
 *      INIT_PROGRESS( <nRow> , <nTotal> [, <cTitle> ] ) -> aProgress
 *
 *  $ARGUMENTS$
 *      nRow       - The row on which the progress bar is to be displayed.
 *      nTotal     - The total number of items for which a progress is
 *                   desired.
 *      cTitle     - The title to be displayed above the progress bar.
 *                   (Default value: "Percentage completed" )
 *
 *  $RETURNS$
 *      aProgress  - Progress information in an array of 4 elements:
 *                   1. nRow      - The row on which the progress bar is
 *                                  to be displayed.
 *                   2. nTotal    - The total number of items for which
 *                                  a progress is desired.
 *                   3. nLast     - The number of the last item processed,
 *                                  initialised to 0.
 *                   4. cScr      - The area of the screen used to display
 *                                  the progress screen before the progress
 *                                  information was displayed.
 *
 *  $DESCRIPTION$
 *      Initialise progress display data, and save screen information.
 *
 *  $EXAMPLES$
 *      aProgress := INIT_PROGRESS( MAXROW() - 3, LASTREC(), "Records changed" )
 *      FOR nRecNo := 1 TO LASTREC()
 *          DBGOTO( nRecNo )
 *          PROGRESS( aProgress )
 *          .
 *          * Do whatever
 *          .
 *       NEXT
 *       KILL_PROGRESS( aWindow )
 *
 *  $SEEALSO$
 *      PROGRESS()
 *      KILL_PROGRESS()
 *
 *  $INCLUDE$
 *      common.ch
 *      box.ch
 *
 *  $END$
 */
FUNCTION INIT_PROGRESS( nRow, nTotal, cTitle )
    LOCAL nTop := nRow - 3
    LOCAL nBtm := nRow + 1
    LOCAL cScr := SAVESCREEN( nTop, 0, nBtm, MAXCOL() )
    LOCAL cPct := "0" + REPLICATE( "-", MAXCOL() - 5 ) + "100"
    DEFAULT cTitle TO "Percentage completed"
    @ nTop, 0, nBtm, MAXCOL() BOX B_DOUBLE + " " COLOR PRO_C_BOX
    @ nTop+1,1 SAY PADC( cTitle, MAXCOL() - 1 ) COLOR PRO_C_BOX
    cPct := STUFF( cPct, INT( LEN( cPct ) / 2 ) - 1, 2, "50" )
    @ nTop+2,1 SAY cPct COLOR PRO_C_BOX
RETURN( { nRow, nTotal, 0, cScr } )
* ======================================================================== *
/*  $DOC$
 *  $FUNCNAME$
 *      PROGRESS()
 *
 *  $ONELINER$
 *      Display progress data.
 *
 *  $SYNTAX$
 *      PROGRESS( <aProgress> )
 *
 *  $ARGUMENTS$
 *      aProgress  - Progress information in an array of 4 elements:
 *                   1. nRow      - The row on which the progress bar is
 *                                  to be displayed.
 *                   2. nTotal    - The total number of items for which
 *                                  a progress is desired.
 *                   3. nLast     - The number of the last item processed,
 *                                  incremented before use.
 *                   4. cScr      - The area of the screen used to display
 *                                  the progress screen before the progress
 *                                  information was displayed.
 *                                  (Not used in this function.)
 *
 *  $RETURNS$
 *      NIL
 *
 *  $DESCRIPTION$
 *      Display progress data.
 *
 *  $EXAMPLES$
 *      aProgress := INIT_PROGRESS( MAXROW() - 3, LASTREC(), "Records changed" )
 *      FOR nRecNo := 1 TO LASTREC()
 *          DBGOTO( nRecNo )
 *          PROGRESS( aProgress )
 *          .
 *          * Do whatever
 *          .
 *       NEXT
 *       KILL_PROGRESS( aWindow )
 *
 *  $SEEALSO$
 *      INIT_PROGRESS()
 *      KILL_PROGRESS()
 *
 *  $INCLUDE$
 *
 *  $END$
 */
FUNCTION PROGRESS( aProgress )
    LOCAL nOldLen  := INT( ( aProgress[ PRO_DUN ] / aProgress[ PRO_TTL ] ) * ( MAXCOL() - 1 ) )
    LOCAL nNewLen  := INT( ( ++aProgress[ PRO_DUN ] / aProgress[ PRO_TTL ] ) * ( MAXCOL() - 1 ) )
    IF aProgress[ PRO_DUN ] >= aProgress[ PRO_TTL ]
        @ aProgress[ PRO_ROW ],1 SAY REPLICATE( "", MAXCOL() - 1 ) COLOR PRO_C_LYN
        ALLKEY( 1 )
    ELSE
        IF nNewLen > nOldLen
            @ aProgress[ PRO_ROW ],1 SAY REPLICATE( "", nNewLen ) COLOR PRO_C_LYN
        ENDIF
    ENDIF
RETURN( NIL )
* ======================================================================== *
/*  $DOC$
 *  $FUNCNAME$
 *      KILL_PROGRESS()
 *
 *  $ONELINER$
 *      Restore screen to pre-progress display condition.
 *
 *  $SYNTAX$
 *      KILL_PROGRESS( <aProgress> )
 *
 *  $ARGUMENTS$
 *      aProgress  - Progress information in an array of 4 elements:
 *                   1. nRow      - The row on which the progress bar is
 *                                  to be displayed.
 *                   2. nTotal    - The total number of items for which
 *                                  a progress is desired.
 *                                  (Not used in this function.)
 *                   3. nLast     - The number of the last item processed,
 *                                  incremented before use.
 *                                  (Not used in this function.)
 *                   4. cScr      - The area of the screen used to display
 *                                  the progress screen before the progress
 *                                  information was displayed.
 *
 *  $RETURNS$
 *      NIL
 *
 *  $DESCRIPTION$
 *      Restore screen to pre-progress display condition.
 *
 *  $EXAMPLES$
 *      aProgress := INIT_PROGRESS( MAXROW() - 3, LASTREC(), "Records changed" )
 *      FOR nRecNo := 1 TO LASTREC()
 *          DBGOTO( nRecNo )
 *          PROGRESS( aProgress )
 *          .
 *          * Do whatever
 *          .
 *       NEXT
 *       KILL_PROGRESS( aWindow )
 *
 *  $SEEALSO$
 *      INIT_PROGRESS()
 *      PROGRESS()
 *
 *  $INCLUDE$
 *
 *  $END$
 */
FUNCTION KILL_PROGRESS( aProgress )
    LOCAL nTop := aProgress[ PRO_ROW ] - 3
    LOCAL nBtm := aProgress[ PRO_ROW ] + 1
    RESTSCREEN( nTop, 0, nBtm, MAXCOL(), aProgress[ PRO_SCR ] )
RETURN( NIL )

